import 'package:flutter/material.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:just_audio/just_audio.dart';
import 'package:move_to_background/move_to_background.dart';

import '../sound/sound.dart';
import 'app_config.dart';
import 'app_locale.dart';
import 'app_opener.dart';
import 'app_router.dart';
import 'session_amap.dart';
import 'widget/widget_badge.dart';
import 'widget/widget_dialog.dart';

///
///应用主页
///
class AppHome extends StatefulWidget {
  const AppHome({Key? key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _AppHomeState();
}

class _AppHomeState extends State<AppHome> {
  final _appOpener = AppOpener();
  final _ringPlayer = AudioPlayer();
  bool _socketInited = false;
  int _currentIndex = 0;

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['AppHome.initState()']);
    AppConfig.client.connect(onopen: _onSocketOpen, onclose: _onSocketClose, onerror: _onSocketError, onretry: _onSocketRetry, onheart: _onSocketHeart);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
    AppConfig.client.setOnMessageSendListener(_onMessageSendListener, remove: false);
    _appOpener.init(onAmapLocationOk: _onAmapLocationOk);
    _ringPlayer.setAsset(SoundAssets.messageRing2);
  }

  @override
  void dispose() {
    AppConfig.logDebug(['AppHome.dispose()']);
    AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
    AppConfig.client.setOnMessageSendListener(_onMessageSendListener, remove: true);
    AppConfig.client.release();
    _appOpener.dispose();
    _ringPlayer.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    if (!_socketInited) {
      return AppRouter.createAppSplashWidget();
    }
    final Widget bodyWidget;
    switch (_currentIndex) {
      case 0:
        bodyWidget = AppRouter.createAppHomeSessionHomeWidget();
        break;
      case 1:
        bodyWidget = AppRouter.createAppHomeCustomOneWidget();
        break;
      case 2:
        bodyWidget = AppRouter.createAppHomeCustomTwoWidget();
        break;
      case 3:
        bodyWidget = AppRouter.createAppHomeUsermeHomeWidget();
        break;
      default:
        bodyWidget = Center(child: Text('$_currentIndex'));
        break;
    }
    return WillPopScope(
      child: Scaffold(
        body: bodyWidget,
        bottomNavigationBar: BottomNavigationBar(
          backgroundColor: _currentIndex == 0 ? null : themeData.cardColor,
          items: [
            BottomNavigationBarItem(
              icon: WidgetBadge(icon: const Icon(Icons.sms_outlined), count: AppConfig.client.sessionState.unread + AppConfig.client.waitshipState.unread),
              activeIcon: WidgetBadge(icon: const Icon(Icons.sms), count: AppConfig.client.sessionState.unread + AppConfig.client.waitshipState.unread),
              label: AppLocalizations.appHomeTabItemSession,
            ),
            BottomNavigationBarItem(
              icon: const Icon(Icons.grid_view_outlined),
              activeIcon: const Icon(Icons.grid_view_sharp),
              label: AppLocalizations.appHomeTabItemCustom1,
            ),
            BottomNavigationBarItem(
              icon: const Icon(Icons.pages_outlined),
              activeIcon: const Icon(Icons.pages),
              label: AppLocalizations.appHomeTabItemCustom2,
            ),
            BottomNavigationBarItem(
              icon: WidgetBadge(icon: const Icon(Icons.account_circle_outlined), dotOnly: true, count: AppConfig.countUserNotSettings()),
              activeIcon: WidgetBadge(icon: const Icon(Icons.account_circle), dotOnly: true, count: AppConfig.countUserNotSettings()),
              label: AppLocalizations.appHomeTabItemUserme,
            )
          ],
          currentIndex: _currentIndex,
          onTap: (index) {
            if (mounted) {
              setState(() {
                _currentIndex = index;
              });
            }
          },
        ),
      ),
      onWillPop: () async {
        await MoveToBackground.moveTaskToBack(); //后台运行
        return false; //拦截返回按钮
      },
    );
  }

  void _onSocketOpen() {
    AppConfig.client.userEnter().then((value) {
      if (value.ok) {
        if (mounted) {
          //定位一次
          _appOpener.startAmapLocation(onceLocation: true);
          //刷新界面
          setState(() {
            _socketInited = true;
          });
        }
      } else if (value.code == 401) {
        if (!mounted) return;
        //登录已经失效
        WidgetDialog.alert(
          context,
          body: AppLocalizations.appHomeUserExpiredHint,
          rightOnly: true,
          rightText: AppLocalizations.appHomeUserExpiredSure,
          rightColor: Colors.redAccent,
        ).then((value) async {
          //下面操作必须进行
          AppConfig.clearUserCredentials();
          AppRouter.pushAppLoginNormalPage(context);
        });
      }
    });
  }

  void _onSocketClose(int code, String reason) {
    _appOpener.stopAmapLocation();
  }

  void _onSocketError(String error) {}

  void _onSocketRetry(int count) {}

  void _onSocketHeart(int second, int delay) {}

  void _onAmapLocationOk(Location location) {
    final user = UserDirty();
    user.location = location;
    AppConfig.client.userUpdate(uid: AppConfig.client.user.id, fields: user.data);
  }

  void _clientCommonListener(EasyPacket packet) {
    if (mounted) {
      setState(() {});
    }
  }

  void _onMessageSendListener(EasyPacket packet) async {
    if (mounted) {
      setState(() {});
    }
    final message = AppConfig.client.parseMessageFromSendOrUpdate(packet);
    String shipNick = message.sid.toHexString();
    bool shipNotice = false;
    if (message.from == Constant.msgFromUser) {
      final ship = AppConfig.client.parseUserShipFromSendOrUpdate(packet);
      final user = AppConfig.client.getUser(ship.rid);
      shipNick = ComTools.formatUserShipNick(ship, user);
      shipNotice = ship.notice;
    } else if (message.from == Constant.msgFromTeam) {
      final ship = AppConfig.client.parseTeamShipFromSendOrUpdate(packet);
      final team = AppConfig.client.getTeam(ship.rid);
      shipNick = ComTools.formatTeamNick(team);
      shipNotice = ship.notice;
    }
    if (AppConfig.runBackground) {
      if (AppConfig.client.user.notice && shipNotice && message.uid != AppConfig.client.user.id) {
        AppConfig.showNotification(
          title: shipNick,
          body: message.short,
          groupKey: message.sid.toHexString(),
          androidChannelId: AppLocalizations.androidNotificationChannelMessageId,
          androidChannelName: AppLocalizations.androidNotificationChannelMessageName,
          androidChannelDescription: AppLocalizations.androidNotificationChannelMessageDesc,
          payload: message.toString(),
        );
      }
    } else {
      switch (message.type) {
        case Constant.msgTypeRealtimeVoice:
        case Constant.msgTypeRealtimeVideo:
        case Constant.msgTypeRealtimeShare:
          AppRouter.pushSessionRealtimePage(context, message: message); //实时媒体消息
          break;
        case Constant.msgTypeRealtimeLocal:
          AppRouter.pushSessionAmapPage(context, type: SessionAmapType.realtimeShare); //实时位置消息
          break;
        default:
          if (!AppConfig.client.user.silent && shipNotice && message.uid != AppConfig.client.user.id) {
            await _ringPlayer.seek(Duration.zero);
            _ringPlayer.play();
          }
      }
    }
  }
}
