import 'dart:convert';
import 'dart:math' as math;
import 'package:flutter/material.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:webview_flutter/webview_flutter.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_dialog.dart';
import 'widget/widget_input.dart';
import 'widget/widget_sheet.dart';

///
///网络视图页类型
///
enum AppWebViewType {
  ///普通
  normal,
}

///
///网络视图页
///
class AppWebView extends StatefulWidget {
  ///类型
  final AppWebViewType type;

  ///标题
  final String? title;

  ///链接
  final String? url;

  ///内容
  final String? body;

  const AppWebView({
    Key? key,
    required this.type,
    this.title,
    this.url,
    this.body,
  })  : assert(url != null || body != null),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _AppWebViewState();
}

enum _AppWebViewMenuItem { share, report, reload }

class _AppWebViewState extends State<AppWebView> {
  WebViewController? _webViewController;

  String _pageTitle = '';

  //网页的分享参数
  String _shareTitle = '';
  String _shareBody = '';
  String _shareIconUrl = '';
  String _shareLinkUrl = '';

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['AppWebView.initState()', widget.type]);
    _pageTitle = widget.title ?? '';
  }

  @override
  void dispose() {
    AppConfig.logDebug(['AppWebView.dispose()', widget.type]);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    // final themeData = Theme.of(context);
    // final queryData = MediaQuery.of(context);
    return Scaffold(
      appBar: AppBar(
        title: Text(_pageTitle),
        actions: [
          IconButton(
            icon: const Icon(Icons.more_horiz_outlined),
            tooltip: AppLocalizations.appWebViewActionBtnMenuTip,
            onPressed: () => _buildAppBarMenuItem([_AppWebViewMenuItem.share, _AppWebViewMenuItem.report, _AppWebViewMenuItem.reload]),
          ),
        ],
      ),
      body: WebView(
        debuggingEnabled: false,
        userAgent: AppConfig.agentInfo,
        backgroundColor: Colors.transparent,
        initialUrl: widget.url,
        javascriptMode: JavascriptMode.unrestricted,
        initialMediaPlaybackPolicy: AutoMediaPlaybackPolicy.always_allow,
        allowsInlineMediaPlayback: true,
        gestureNavigationEnabled: true,
        zoomEnabled: true,
        initialCookies: const [],
        gestureRecognizers: null,
        onWebViewCreated: (webViewController) {
          _webViewController = webViewController;
        },
        onPageStarted: (url) {
          //在这个函数里面不要执行刷新页面，否则导致页面加载两次，使得userAgent设置不生效
          AppConfig.logDebug(['onPageStarted:', url]);
          _shareTitle = url;
          _shareBody = url;
          _shareIconUrl = '';
          _shareLinkUrl = url;
        },
        onProgress: (progress) {
          AppConfig.logDebug(['onProgress:', progress]);
        },
        onPageFinished: (url) {
          AppConfig.logDebug(['onPageFinished:', url]);
          _webViewController?.getTitle().then((value) {
            if (mounted) {
              setState(() {
                _pageTitle = _shareTitle = value ?? url;
              });
            }
          });
        },
        navigationDelegate: (request) {
          return NavigationDecision.navigate;
        },
        javascriptChannels: {
          _javascriptChannelToast(context),
          _javascriptChannelAlert(context),
          _javascriptChannelShare(context),
          _javascriptChannelInput(context),
        },
        onWebResourceError: (error) {
          AppConfig.logError(['onWebResourceError', error]);
        },
      ),
    );
  }

  void _buildAppBarMenuItem(List<_AppWebViewMenuItem> menuItems) {
    final itemDatas = <WidgetSheetItem>[];
    for (var element in menuItems) {
      switch (element) {
        case _AppWebViewMenuItem.share:
          itemDatas.add(WidgetSheetItem(0, AppLocalizations.appWebViewMenuItemShareToUser));
          itemDatas.add(WidgetSheetItem(1, AppLocalizations.appWebViewMenuItemShareToTeam));
          break;
        case _AppWebViewMenuItem.report:
          itemDatas.add(WidgetSheetItem(2, AppLocalizations.appWebViewMenuItemReport));
          break;
        case _AppWebViewMenuItem.reload:
          itemDatas.add(WidgetSheetItem(3, AppLocalizations.appWebViewMenuItemReload));
          break;
      }
    }
    WidgetSheet.showBottomTextSheet(context, itemDatas, cancelText: AppLocalizations.appWebViewMenuItemCancel).then((value) async {
      switch (value) {
        case 0:
          AppRouter.pushContactUserShipsPickerPage(context, title: AppLocalizations.appWebViewMenuItemShareToUser).then((value) {
            if (value == null) return;
            for (var element in value) {
              AppConfig.client.messageSendShareHtmlPage(
                sid: element.sid,
                from: Constant.msgFromUser,
                title: _shareTitle,
                body: _shareBody,
                shareIconUrl: _shareIconUrl,
                shareLinkUrl: _shareLinkUrl,
              );
            }
            SmartDialog.showToast(AppLocalizations.appWebViewPageShareFinished);
          });
          break;
        case 1:
          AppRouter.pushContactTeamShipsPickerPage(context, title: AppLocalizations.appWebViewMenuItemShareToTeam).then((value) {
            if (value == null) return;
            for (var element in value) {
              AppConfig.client.messageSendShareHtmlPage(
                sid: element.sid,
                from: Constant.msgFromTeam,
                title: _shareTitle,
                body: _shareBody,
                shareIconUrl: _shareIconUrl,
                shareLinkUrl: _shareLinkUrl,
              );
            }
            SmartDialog.showToast(AppLocalizations.appWebViewPageShareFinished);
          });
          break;
        case 2:
          break;
        case 3:
          _webViewController?.reload();
          break;
      }
    });
  }

  JavascriptChannel _javascriptChannelToast(BuildContext context) {
    return JavascriptChannel(
      name: '_javascriptChannelToast',
      onMessageReceived: (JavascriptMessage data) {
        try {
          final Map<String, dynamic> params = json.decode(data.message);
          final String text = params['text'] ?? 'Toast';
          final int mills = math.max(params['mills'] ?? 2000, 100);
          final String align = params['align'] ?? 'bottomCenter';
          switch (align) {
            case 'topCenter':
              SmartDialog.showToast(text, displayTime: Duration(milliseconds: mills), alignment: Alignment.topCenter);
              break;
            case 'center':
              SmartDialog.showToast(text, displayTime: Duration(milliseconds: mills), alignment: Alignment.center);
              break;
            case 'bottomCenter':
              SmartDialog.showToast(text, displayTime: Duration(milliseconds: mills), alignment: Alignment.bottomCenter);
              break;
          }
        } catch (error, stack) {
          AppConfig.logError([error, '\n', stack]);
        }
      },
    );
  }

  JavascriptChannel _javascriptChannelAlert(BuildContext context) {
    return JavascriptChannel(
      name: '_javascriptChannelAlert',
      onMessageReceived: (JavascriptMessage data) {
        try {
          final Map<String, dynamic> params = json.decode(data.message);
          final String? title = params['title'];
          final String body = params['body'] ?? 'Alert';
          final String? leftText = params['leftText'];
          final String? rightText = params['rightText'];
          final int? leftColor = params['leftColor'];
          final int? rightColor = params['rightColor'];
          final bool rightOnly = params['rightOnly'] ?? false;
          final String? callbackId = params['callbackId'];
          WidgetDialog.alert(
            context,
            title: title,
            body: body,
            leftText: leftText,
            rightText: rightText,
            leftColor: leftColor == null ? null : Color(leftColor),
            rightColor: rightColor == null ? null : Color(rightColor),
            rightOnly: rightOnly,
          ).then((value) {
            if (callbackId != null) {
              final result = {'id': callbackId, 'ok': value ?? false};
              _webViewController?.runJavascript('WebViewApi.alertCallback(${json.encode(result)})');
            }
          });
        } catch (error, stack) {
          AppConfig.logError([error, '\n', stack]);
        }
      },
    );
  }

  JavascriptChannel _javascriptChannelShare(BuildContext context) {
    return JavascriptChannel(
      name: '_javascriptChannelShare',
      onMessageReceived: (JavascriptMessage data) async {
        try {
          Map<String, dynamic> params = json.decode(data.message); //分享
          _shareTitle = params['title'] ?? _shareTitle;
          _shareBody = params['body'] ?? _shareBody;
          _shareIconUrl = params['icon'] ?? _shareIconUrl;
          _shareLinkUrl = params['link'] ?? _shareLinkUrl;
          final String? callbackId = params['callbackId'];
          if (callbackId != null) {
            final result = {'id': callbackId, 'ok': true, 'msg': 'success'};
            _webViewController?.runJavascript('WebViewApi.shareCallback(${json.encode(result)})');
          }
        } catch (error, stack) {
          AppConfig.logError([error, '\n', stack]);
        }
      },
    );
  }

  JavascriptChannel _javascriptChannelInput(BuildContext context) {
    return JavascriptChannel(
      name: '_javascriptChannelInput',
      onMessageReceived: (JavascriptMessage data) {
        try {
          final Map<String, dynamic> params = json.decode(data.message);
          final String? labelText = params['labelText'];
          final String? helperText = params['helperText'];
          final int maxLength = math.max(params['maxLength'] ?? 16, 1);
          final int maxLines = math.max(math.min(params['maxLines'] ?? 1, 4), 1);
          final String defValue = params['defValue'] ?? '';
          final String inputType = params['inputType'] ?? '';
          final String? callbackId = params['callbackId'];
          Navigator.push<String?>(context, WidgetInput(labelText: labelText, helperText: helperText, maxLength: maxLength, maxLines: maxLines, defValue: defValue, inputType: inputType)).then((value) {
            if (callbackId != null) {
              final result = {'id': callbackId, 'ok': value != null, 'val': value};
              _webViewController?.runJavascript('WebViewApi.inputCallback(${json.encode(result)})');
            }
          });
        } catch (error, stack) {
          AppConfig.logError([error, '\n', stack]);
        }
      },
    );
  }
}
