import 'package:flutter/material.dart';
import 'package:flutter_slidable/flutter_slidable.dart';
import 'package:flutter_smart_dialog/flutter_smart_dialog.dart';
import 'package:imsdk_client_core/imsdk_client_core.dart';
import 'package:scrollable_positioned_list/scrollable_positioned_list.dart';

import 'app_config.dart';
import 'app_locale.dart';
import 'app_router.dart';
import 'widget/widget_avator.dart';
import 'widget/widget_divider.dart';
import 'widget/widget_letters.dart';

///
/// 通用列表页类型
///
enum ContactAZListType {
  ///好友申请列表
  waitships,

  ///好友列表
  userships,

  ///好友列表选择器
  usershipsPicker,

  ///群组列表
  teamships,

  ///群组列表选择器
  teamshipsPicker,

  ///群组成员列表
  _teamusers,

  ///群组成员列表选择器
  _teamusersPicker,
}

///
/// 通用列表页
///
class ContactAZList extends StatefulWidget {
  /* ******************************** instance content ******************************** */
  ///页面类型
  final ContactAZListType type;

  ///页面标题
  final String? title;

  ///关联群组
  final Team? team;

  const ContactAZList({Key? key, required this.type, this.title, this.team}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _ContactAZListState();

  /* ******************************** static content ******************************** */
  ///群组成员路由单例
  static MaterialPageRoute<Set<TeamShip>?>? _teamusersRouteInstance;

  ///为群组成员创建单例
  static Future<Set<TeamShip>?> pushRouteSingletonForTeamUsers(BuildContext context, String title, Team team, bool picker) {
    _removeTeamUsersRouteInstance(context);
    _teamusersRouteInstance = MaterialPageRoute(builder: (context) => ContactAZList(type: picker ? ContactAZListType._teamusersPicker : ContactAZListType._teamusers, title: title, team: team));
    return Navigator.push<Set<TeamShip>?>(context, _teamusersRouteInstance!);
  }

  ///移除当前单例
  static void _removeTeamUsersRouteInstance(BuildContext context) {
    if (_teamusersRouteInstance != null && _teamusersRouteInstance!.isActive) {
      Navigator.removeRoute(context, _teamusersRouteInstance!);
      _teamusersRouteInstance = null;
    }
  }
}

class _ContactAZListState extends State<ContactAZList> {
  final _scrollController = ItemScrollController();
  final _letters = List<String>.from(NetClientAzState.letters);
  final _checkedUserShips = <UserShip>{};
  final _checkedTeamShips = <TeamShip>{};

  @override
  void initState() {
    super.initState();
    AppConfig.logDebug(['ContactAZList.initState()', widget.type]);
    switch (widget.type) {
      case ContactAZListType.waitships:
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: false);
        AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: false);
        break;
      case ContactAZListType.userships:
      case ContactAZListType.usershipsPicker:
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: false);
        AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: false);
        break;
      case ContactAZListType.teamships:
      case ContactAZListType.teamshipsPicker:
        AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: false);
        AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
        break;
      case ContactAZListType._teamusers:
      case ContactAZListType._teamusersPicker:
        _letters.insert(0, NetClientAzState.header);
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: false);
        AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: false);
        AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: false);
        AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: false);
        _loadTeamMember();
        break;
    }
  }

  @override
  void dispose() {
    AppConfig.logDebug(['ContactAZList.dispose()', widget.type]);
    switch (widget.type) {
      case ContactAZListType.waitships:
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: true);
        AppConfig.client.setOnWaitShipUpdateListener(_clientCommonListener, remove: true);
        break;
      case ContactAZListType.userships:
      case ContactAZListType.usershipsPicker:
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: true);
        AppConfig.client.setOnUserShipUpdateListener(_clientCommonListener, remove: true);
        break;
      case ContactAZListType.teamships:
      case ContactAZListType.teamshipsPicker:
        AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: true);
        AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
        break;
      case ContactAZListType._teamusers:
      case ContactAZListType._teamusersPicker:
        AppConfig.client.setOnUserFetchedWatcher(_clientCommonListener, remove: true);
        AppConfig.client.setOnTeamFetchedWatcher(_clientCommonListener, remove: true);
        AppConfig.client.setOnTeamUpdateListener(_clientCommonListener, remove: true);
        AppConfig.client.setOnTeamShipUpdateListener(_clientCommonListener, remove: true);
        break;
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Scaffold(
      resizeToAvoidBottomInset: false,
      backgroundColor: themeData.cardColor,
      appBar: _buildAppBar(themeData),
      body: Column(
        children: [
          WidgetDivider.forFull,
          Expanded(child: _buildBody(themeData)),
          WidgetDivider.forFull,
        ],
      ),
    );
  }

  AppBar? _buildAppBar(ThemeData themeData) {
    switch (widget.type) {
      case ContactAZListType.usershipsPicker:
        return AppBar(
          backgroundColor: themeData.cardColor,
          title: Text(widget.title ?? ''),
          actions: [
            TextButton(
              onPressed: _checkedUserShips.isEmpty ? null : () => Navigator.pop(context, _checkedUserShips),
              child: Text('${AppLocalizations.contactAZListActionBtnSure}(${_checkedUserShips.length})'),
            ),
          ],
        );
      case ContactAZListType.teamshipsPicker:
        return AppBar(
          backgroundColor: themeData.cardColor,
          title: Text(widget.title ?? ''),
          actions: [
            TextButton(
              onPressed: _checkedTeamShips.isEmpty ? null : () => Navigator.pop(context, _checkedTeamShips),
              child: Text('${AppLocalizations.contactAZListActionBtnSure}(${_checkedTeamShips.length})'),
            ),
          ],
        );
      case ContactAZListType._teamusers:
        if (ComTools.isTeamAdmin(widget.team!, AppConfig.client.user.id)) {
          return AppBar(
            backgroundColor: themeData.cardColor,
            title: Text(widget.title ?? ''),
            actions: [
              IconButton(
                icon: const Icon(Icons.add),
                tooltip: AppLocalizations.contactAZListActionBtnPlusTip,
                onPressed: () {
                  AppRouter.pushContactUserShipsPickerPage(context, title: AppLocalizations.contactAZListActionBtnPlusTip, team: widget.team).then((value) async {
                    if (value == null) return;
                    if (!mounted) return; //此时可能已经被移出该群组了
                    SmartDialog.showLoading(msg: AppLocalizations.contactAZListTeamUsersActionLoading);
                    final result = await AppConfig.client.teamShipPass(tid: widget.team!.id, uids: value.map((e) => e.rid).toList());
                    SmartDialog.dismiss();
                    if (result.ok) {
                      _loadTeamMember();
                    }
                    SmartDialog.showToast(result.desc);
                  });
                },
              ),
            ],
          );
        } else {
          return AppBar(
            backgroundColor: themeData.cardColor,
            title: Text(widget.title ?? ''),
          );
        }
      case ContactAZListType._teamusersPicker:
        return AppBar(
          backgroundColor: themeData.cardColor,
          title: Text(widget.title ?? ''),
          actions: [
            TextButton(
              onPressed: _checkedTeamShips.isEmpty ? null : () => Navigator.pop(context, _checkedTeamShips),
              child: Text('${AppLocalizations.contactAZListActionBtnSure}(${_checkedTeamShips.length})'),
            ),
          ],
        );
      default:
        return null;
    }
  }

  Widget _buildBody(ThemeData themeData) {
    final isSelectMode = widget.type == ContactAZListType.usershipsPicker || widget.type == ContactAZListType.teamshipsPicker || widget.type == ContactAZListType._teamusersPicker;
    late NetClientAzState azDataState;
    late String totalSuffix;
    switch (widget.type) {
      case ContactAZListType.waitships:
        totalSuffix = AppLocalizations.contactAZListSuffixOfWaitships;
        azDataState = AppConfig.client.waitshipState;
        break;
      case ContactAZListType.userships:
      case ContactAZListType.usershipsPicker:
        totalSuffix = AppLocalizations.contactAZListSuffixOfUserships;
        azDataState = AppConfig.client.usershipState;
        break;
      case ContactAZListType.teamships:
      case ContactAZListType.teamshipsPicker:
        totalSuffix = AppLocalizations.contactAZListSuffixOfTeamships;
        azDataState = AppConfig.client.teamshipState;
        break;
      case ContactAZListType._teamusers:
      case ContactAZListType._teamusersPicker:
        totalSuffix = AppLocalizations.contactAZListSuffixOfTeamusers;
        azDataState = AppConfig.client.getTeamuserState(widget.team!.id);
        break;
    }
    final dataList = widget.type == ContactAZListType.waitships ? azDataState.okList : azDataState.azList;
    return Stack(
      children: [
        ScrollablePositionedList.builder(
          physics: AppConfig.bodyGlobalScrollPhysics,
          itemScrollController: _scrollController,
          itemCount: dataList.length,
          itemBuilder: (context, index) {
            final item = dataList[index];
            final nextIsData = index + 1 < dataList.length ? (dataList[index + 1] is UserShip || dataList[index + 1] is TeamShip) : false;
            if (item is String) {
              return Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  Container(
                    color: themeData.scaffoldBackgroundColor,
                    padding: AppConfig.bodyGlobalPadding.copyWith(top: AppConfig.bodyGlobalPadding.top / 2, bottom: AppConfig.bodyGlobalPadding.bottom / 2),
                    child: Text(item, textAlign: TextAlign.left, style: themeData.textTheme.caption),
                  ),
                  WidgetDivider.forFull,
                ],
              );
            } else if (item is UserShip) {
              if (widget.type == ContactAZListType.waitships) {
                return Column(
                  children: [
                    ListTile(
                      leading: WidgetAvator(type: WidgetAvatorType.middle, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                      title: Row(
                        children: [
                          Expanded(child: Text(item.displayNick, maxLines: 1, overflow: TextOverflow.ellipsis)),
                          Text(ComTools.formatDateTime(item.time), style: themeData.textTheme.caption),
                        ],
                      ),
                      subtitle: Row(
                        children: [
                          Expanded(child: Text(ComTools.formatWaitShipApply(item, empty: AppLocalizations.contactAZListWaitshipsApplyEmpty), style: themeData.textTheme.caption, maxLines: 1, overflow: TextOverflow.ellipsis)),
                          Text(AppLocalizations.modelConstMapText(item.state), style: themeData.textTheme.caption),
                        ],
                      ),
                      onTap: () => AppRouter.pushContactUserInfoPage(context, uid: item.uid, fid: item.fid, from: item.from),
                    ),
                    nextIsData ? WidgetDivider.forListTileLeadingAvatorMiddle : WidgetDivider.forFull,
                  ],
                );
              }
              return Column(
                children: [
                  ListTile(
                    leading: WidgetAvator(type: WidgetAvatorType.small, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                    title: Text(item.displayNick, overflow: TextOverflow.ellipsis),
                    trailing: isSelectMode ? _buildSelectModeTailing(themeData, item) : null,
                    onTap: isSelectMode ? () => _onSelectModeItemTap(item) : () => AppRouter.pushContactUserInfoPage(context, uid: item.rid, fid: item.fid, from: item.from),
                  ),
                  nextIsData ? WidgetDivider.forListTileLeadingAvatorSmall : WidgetDivider.forFull,
                ],
              );
            } else if (item is TeamShip) {
              if (widget.type == ContactAZListType._teamusers) {
                return Column(
                  children: [
                    Slidable(
                      endActionPane: _buildTeamUserActionPane(themeData, item),
                      child: ListTile(
                        leading: WidgetAvator(type: WidgetAvatorType.small, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                        title: Text(item.displayNick, overflow: TextOverflow.ellipsis),
                        trailing: _buildTeamUserTailing(themeData, item),
                        onTap: () => AppRouter.pushContactUserInfoPage(context, uid: item.uid, fid: item.rid, from: Constant.shipFromMember),
                      ),
                    ),
                    nextIsData ? WidgetDivider.forListTileLeadingAvatorSmall : WidgetDivider.forFull,
                  ],
                );
              }
              if (widget.type == ContactAZListType._teamusersPicker) {
                return Column(
                  children: [
                    ListTile(
                      leading: WidgetAvator(type: WidgetAvatorType.small, url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                      title: Text(item.displayNick, overflow: TextOverflow.ellipsis),
                      trailing: _buildSelectModeTailing(themeData, item),
                      onTap: () => _onSelectModeItemTap(item),
                    ),
                    nextIsData ? WidgetDivider.forListTileLeadingAvatorSmall : WidgetDivider.forFull,
                  ],
                );
              }
              return Column(
                children: [
                  ListTile(
                    leading: WidgetAvator(type: WidgetAvatorType.small, single: ComTools.isTeamShipFromSystem(item), url: item.displayIcon, urlList: item.displayHead, onFormatUrl: AppConfig.formatAttachUrl),
                    title: Text(item.displayNick, overflow: TextOverflow.ellipsis),
                    trailing: isSelectMode ? _buildSelectModeTailing(themeData, item) : null,
                    onTap: isSelectMode ? () => _onSelectModeItemTap(item) : () => AppRouter.pushContactTeamInfoPage(context, tid: item.rid, fid: item.fid, from: item.from),
                  ),
                  nextIsData ? WidgetDivider.forListTileLeadingAvatorSmall : WidgetDivider.forFull,
                ],
              );
            } else if (item is int) {
              return ListTile(
                dense: true,
                title: Text('$item$totalSuffix', style: themeData.textTheme.caption, textAlign: TextAlign.center),
              );
            } else {
              return const SizedBox();
            }
          },
        ),
        Padding(
          padding: const EdgeInsets.symmetric(vertical: WidgetLetters.bubbleRadiusDefault),
          child: widget.type == ContactAZListType.waitships
              ? const SizedBox()
              : WidgetLetters(
                  letters: _letters,
                  onDrag: (char) {
                    final index = azDataState.azMap[char];
                    if (index != null && _scrollController.isAttached) {
                      _scrollController.jumpTo(index: index);
                    }
                  },
                ),
        )
      ],
    );
  }

  Widget? _buildSelectModeTailing(ThemeData themeData, Object ship) {
    if (ship is UserShip) {
      //添加群成员，过滤掉已在群组中的好友
      if (widget.team != null && AppConfig.client.getTeamUser(widget.team!.id, ship.rid) != null) {
        return Row(
          mainAxisSize: MainAxisSize.min,
          children: [const Icon(Icons.motion_photos_off_outlined), SizedBox(width: AppConfig.bodyGlobalPadding.left)],
        );
      }
      //过滤掉来源于系统的好友关系
      if (ComTools.isUserShipFromSystem(ship)) {
        return Row(
          mainAxisSize: MainAxisSize.min,
          children: [const Icon(Icons.motion_photos_off_outlined), SizedBox(width: AppConfig.bodyGlobalPadding.left)],
        );
      }
      //正常选择
      return Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Icon(
            _checkedUserShips.contains(ship) ? Icons.check_circle : Icons.circle_outlined,
            color: _checkedUserShips.contains(ship) ? themeData.colorScheme.secondary : null,
          ),
          SizedBox(width: AppConfig.bodyGlobalPadding.left),
        ],
      );
    } else if (ship is TeamShip) {
      //选择群成员，过滤掉自己
      if (widget.type == ContactAZListType._teamusersPicker && ship.uid == AppConfig.client.user.id) {
        return Row(
          mainAxisSize: MainAxisSize.min,
          children: [const Icon(Icons.motion_photos_off_outlined), SizedBox(width: AppConfig.bodyGlobalPadding.left)],
        );
      }
      //过滤掉来源于系统的群组关系
      if (ComTools.isTeamShipFromSystem(ship)) {
        return Row(
          mainAxisSize: MainAxisSize.min,
          children: [const Icon(Icons.motion_photos_off_outlined), SizedBox(width: AppConfig.bodyGlobalPadding.left)],
        );
      }
      //正常选择
      return Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Icon(
            _checkedTeamShips.contains(ship) ? Icons.check_circle : Icons.circle_outlined,
            color: _checkedTeamShips.contains(ship) ? themeData.colorScheme.secondary : null,
          ),
          SizedBox(width: AppConfig.bodyGlobalPadding.left),
        ],
      );
    }
    return null;
  }

  void _onSelectModeItemTap(Object ship) {
    if (ship is UserShip) {
      if (widget.team != null && AppConfig.client.getTeamUser(widget.team!.id, ship.rid) != null) return; //添加群成员，过滤掉已在群组中的好友
      if (ComTools.isUserShipFromSystem(ship)) return; //过滤掉来源于系统的好友关系
      if (mounted) {
        setState(() {
          if (_checkedUserShips.contains(ship)) {
            _checkedUserShips.remove(ship);
          } else {
            _checkedUserShips.add(ship);
          }
        });
      }
    } else if (ship is TeamShip) {
      if (widget.type == ContactAZListType._teamusersPicker && ship.uid == AppConfig.client.user.id) return; //选择群成员，过滤掉自己
      if (ComTools.isTeamShipFromSystem(ship)) return; //过滤掉来源于系统的群组关系
      if (mounted) {
        setState(() {
          if (_checkedTeamShips.contains(ship)) {
            _checkedTeamShips.remove(ship);
          } else {
            _checkedTeamShips.add(ship);
          }
        });
      }
    }
  }

  Widget? _buildTeamUserTailing(ThemeData themeData, TeamShip ship) {
    if (ComTools.isTeamAdmin(widget.team!, ship.uid)) {
      return Row(
        mainAxisSize: MainAxisSize.min,
        children: [
          Text(ComTools.isTeamSuperAdmin(widget.team!, ship.uid) ? AppLocalizations.contactAZListTeamUsersOwnerHint : AppLocalizations.contactAZListTeamUsersAdminHint, style: themeData.textTheme.caption),
          SizedBox(width: AppConfig.bodyGlobalPadding.left),
        ],
      );
    } else {
      return null;
    }
  }

  ActionPane? _buildTeamUserActionPane(ThemeData themeData, TeamShip ship) {
    final targetIsSuperAdmin = ComTools.isTeamSuperAdmin(widget.team!, ship.uid);
    final targetIsNormalAdmin = ComTools.isTeamNormalAdmin(widget.team!, ship.uid);
    if (ComTools.isTeamSuperAdmin(widget.team!, AppConfig.client.user.id) && !targetIsSuperAdmin) {
      return ActionPane(
        motion: AppConfig.slidableScrollMotion,
        children: [
          SlidableAction(
            backgroundColor: targetIsNormalAdmin ? Colors.blueGrey : Colors.blueAccent,
            foregroundColor: Colors.white,
            label: targetIsNormalAdmin ? AppLocalizations.contactAZListTeamUsersBtnAdminN : AppLocalizations.contactAZListTeamUsersBtnAdminY,
            onPressed: (context) async {
              final team = TeamDirty();
              final admin = <ObjectId>[...widget.team!.admin];
              targetIsNormalAdmin ? admin.remove(ship.uid) : admin.add(ship.uid); //ObjectId已经重定义hashCode函数与==操作符，可以直接用remove
              team.admin = admin;
              SmartDialog.showLoading(msg: AppLocalizations.contactAZListTeamUsersActionLoading);
              final result = await AppConfig.client.teamUpdate(tid: widget.team!.id, fields: team.data);
              SmartDialog.dismiss();
              // if (result.ok) {
              //   _loadTeamMember();
              // }
              SmartDialog.showToast(result.desc);
            },
          ),
          SlidableAction(
            backgroundColor: Colors.deepOrange,
            foregroundColor: Colors.white,
            label: ship.uid == AppConfig.client.user.id ? AppLocalizations.contactAZListTeamUsersBtnQuit : AppLocalizations.contactAZListTeamUsersBtnRemove,
            onPressed: (context) async {
              SmartDialog.showLoading(msg: AppLocalizations.contactAZListTeamUsersActionLoading);
              final result = await AppConfig.client.teamShipNone(tid: widget.team!.id, uid: ship.uid);
              SmartDialog.dismiss();
              if (result.ok && ship.uid != AppConfig.client.user.id) {
                _loadTeamMember();
              }
              SmartDialog.showToast(result.desc);
            },
          ),
        ],
      );
    } else if (ComTools.isTeamAdmin(widget.team!, AppConfig.client.user.id)) {
      return ActionPane(
        motion: AppConfig.slidableScrollMotion,
        children: [
          SlidableAction(
            backgroundColor: Colors.deepOrange,
            foregroundColor: Colors.white,
            label: ship.uid == AppConfig.client.user.id ? AppLocalizations.contactAZListTeamUsersBtnQuit : AppLocalizations.contactAZListTeamUsersBtnRemove,
            onPressed: (context) async {
              SmartDialog.showLoading(msg: AppLocalizations.contactAZListTeamUsersActionLoading);
              final result = await AppConfig.client.teamShipNone(tid: widget.team!.id, uid: ship.uid);
              SmartDialog.dismiss();
              if (result.ok && ship.uid != AppConfig.client.user.id) {
                _loadTeamMember();
              }
              SmartDialog.showToast(result.desc);
            },
          ),
        ],
      );
    } else {
      return null;
    }
  }

  void _loadTeamMember() {
    AppConfig.client.teamMember(tid: widget.team!.id).then((value) {
      if (value.ok) {
        if (mounted) {
          setState(() {});
        }
      }
    });
  }

  void _clientCommonListener(EasyPacket packet) {
    if (widget.type == ContactAZListType._teamusers) {
      if (AppConfig.client.getTeamShip(widget.team!.id) == null) {
        ContactAZList._removeTeamUsersRouteInstance(context); //已经不在该群组中了
      } else {
        if (mounted) {
          setState(() {});
        }
      }
    } else {
      if (mounted) {
        setState(() {});
      }
    }
  }
}
