import 'dart:math' as math;
import 'package:extended_image/extended_image.dart';
import 'package:flutter/material.dart';

///
///头像尺寸类型
///
enum WidgetAvatorType {
  ///超大的
  huge,

  ///大的
  large,

  ///中等的
  middle,

  ///小的
  small,
}

///
///头像
///
class WidgetAvator extends StatelessWidget {
  static const hugeTypeRadius = 38.0; //头像半径-巨
  static const largeTypeRadius = 26.0; //头像半径-大
  static const middleTypeRadius = 22.0; //头像半径-中
  static const smallTypeRadius = 20.0; //头像半径-小
  static const _roundedRectRadius = 4.0; //使用矩形头像的圆角半径
  static const _emptyIconSizeRateFather = 1.5; //父级icon尺寸为半径的倍数
  static const _emptyIconSizeRateChild = 1.8; //子级icon尺寸为半径的倍数

  final WidgetAvatorType type;
  final bool rectBg;
  final bool greyBg;
  final bool single;
  final String? url;
  final List<String>? urlList;
  final Widget? child;
  final Color? childColor;
  final String Function(String val) onFormatUrl;
  final double radius;

  const WidgetAvator({
    Key? key,
    this.type = WidgetAvatorType.small,
    this.rectBg = true,
    this.greyBg = true,
    this.single = true,
    this.url,
    this.urlList,
    this.child,
    this.childColor,
    required this.onFormatUrl,
  })  : radius = type == WidgetAvatorType.huge ? hugeTypeRadius : (type == WidgetAvatorType.large ? largeTypeRadius : (type == WidgetAvatorType.middle ? middleTypeRadius : smallTypeRadius)),
        super(key: key);

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final lightColor = greyBg ? (Colors.grey[400] ?? themeData.primaryColorLight) : themeData.primaryColorLight;
    final darkColor = greyBg ? (Colors.grey[700] ?? themeData.primaryColorDark) : themeData.primaryColorDark;
    final fBoxSize = radius * 2;
    final fBoxRadius = rectBg ? _roundedRectRadius : fBoxSize;
    final fBackColor = themeData.brightness == Brightness.light ? lightColor : darkColor;
    final fIconColor = themeData.brightness == Brightness.light ? darkColor : lightColor;
    final fIconSize = radius * WidgetAvator._emptyIconSizeRateFather;
    if (child == null) {
      return Container(
        width: fBoxSize,
        height: fBoxSize,
        clipBehavior: Clip.hardEdge,
        decoration: ShapeDecoration(
          color: fBackColor,
          shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(fBoxRadius))),
        ),
        child: single ? _buildAvatorItem(url, fIconColor, fIconSize) : _buildAvatorGroup(fBoxSize, fBackColor, fIconColor, fIconSize),
      );
    } else {
      return Container(
        width: fBoxSize,
        height: fBoxSize,
        clipBehavior: Clip.hardEdge,
        decoration: ShapeDecoration(
          color: childColor ?? fBackColor,
          shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(fBoxRadius))),
        ),
        child: child,
      );
    }
  }

  Widget _buildAvatorItem(String? url, Color iconColor, double iconSize) {
    return _isValidUrl(url)
        ? ExtendedImage.network(
            onFormatUrl(url!),
            fit: BoxFit.cover,
            loadStateChanged: (imageState) {
              switch (imageState.extendedImageLoadState) {
                case LoadState.loading:
                case LoadState.failed:
                  return Icon(Icons.person, color: iconColor, size: iconSize);
                case LoadState.completed:
                  return null;
              }
            },
          )
        : Icon(Icons.person, color: iconColor, size: iconSize);
  }

  Widget _buildAvatorGroup(double fBoxSize, Color fBackColor, Color fIconColor, double fIconSize) {
    if (urlList == null || urlList!.isEmpty) {
      return Icon(Icons.people, color: fIconColor, size: fIconSize);
    }
    final backColor = fIconColor;
    final iconColor = fBackColor;
    final count = math.min(urlList!.length, 9);
    final children = <Widget>[];
    if (rectBg) {
      final column = count <= 4 ? 2 : 3;
      const margin = _roundedRectRadius / 2;
      final itemSize = (fBoxSize - column * margin - margin) / column;
      final offsetY = count <= 2 ? (fBoxSize - margin * 2 - itemSize) / 2 : 0;
      for (int i = 0; i < count; i++) {
        final urlItem = urlList![i % urlList!.length];
        final col = i % column;
        final row = (i / column).floor();
        children.add(Positioned(
          top: row * (itemSize + margin) + margin + offsetY,
          left: col * (itemSize + margin) + margin,
          child: Container(
            width: itemSize,
            height: itemSize,
            color: backColor,
            child: _buildAvatorItem(urlItem, iconColor, itemSize * 0.5 * WidgetAvator._emptyIconSizeRateChild),
          ),
        ));
      }
    } else {
      double childRadiusC = 0;
      double childRadiusS = 0;
      double startAngle = 0;
      switch (count) {
        case 1:
          childRadiusC = radius / 2;
          break;
        case 2:
          childRadiusS = radius / 2;
          break;
        case 3:
          childRadiusS = radius * math.cos(math.pi / 6) / (1 + math.cos(math.pi / 6));
          startAngle = math.pi / 2;
          break;
        case 4:
          childRadiusS = radius * (math.sqrt(2) - 1);
          break;
        default:
          childRadiusC = radius / 3;
          childRadiusS = radius / 3;
          if (count == 6) startAngle = math.pi / 2;
          break;
      }
      if (childRadiusC > 0) {
        final itemSize = childRadiusC * 2;
        final urlItem = urlList!.first;
        children.add(
          Positioned(
            left: radius - childRadiusC,
            bottom: radius - childRadiusC,
            child: Container(
              width: itemSize,
              height: itemSize,
              clipBehavior: Clip.hardEdge,
              decoration: ShapeDecoration(
                color: backColor,
                shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(itemSize))),
              ),
              child: _buildAvatorItem(urlItem, iconColor, childRadiusC * WidgetAvator._emptyIconSizeRateChild),
            ),
          ),
        );
      }
      if (childRadiusS > 0) {
        final stepAngle = math.pi * 2 / (childRadiusC > 0 ? count - 1 : count);
        final startIndex = childRadiusC > 0 ? 1 : 0;
        for (int i = startIndex; i < count; i++) {
          final itemSize = childRadiusS * 2;
          final urlItem = urlList![i % urlList!.length];
          children.add(
            Positioned(
              left: radius - childRadiusS + (radius - childRadiusS) * math.cos(startAngle + stepAngle * (i - startIndex)),
              bottom: radius - childRadiusS + (radius - childRadiusS) * math.sin(startAngle + stepAngle * (i - startIndex)),
              child: Container(
                width: itemSize,
                height: itemSize,
                clipBehavior: Clip.hardEdge,
                decoration: ShapeDecoration(
                  color: backColor,
                  shape: RoundedRectangleBorder(borderRadius: BorderRadius.all(Radius.circular(itemSize))),
                ),
                child: _buildAvatorItem(urlItem, iconColor, childRadiusS * WidgetAvator._emptyIconSizeRateChild),
              ),
            ),
          );
        }
      }
    }
    return Stack(children: children);
  }

  bool _isValidUrl(String? val) {
    return val != null && val.trim() != '';
  }
}
