import 'package:flutter/material.dart';

///
///字母索引选择器
///
class WidgetLetters extends StatefulWidget {
  static const bubbleRadiusDefault = 33.0;

  final List<String> letters;
  final double sliderWidth;
  final Color sliderColor;
  final double bubbleWidth;
  final double bubbleHeight;
  final double bubbleRadius;
  final double bubbleTextSize;
  final Color bubbleTextColor;
  final double bubbleTailOffsetX;
  final void Function(String char) onDrag;

  const WidgetLetters({
    Key? key,
    required this.letters,
    this.sliderWidth = 30,
    this.sliderColor = Colors.transparent,
    this.bubbleWidth = bubbleRadiusDefault * 3,
    this.bubbleHeight = bubbleRadiusDefault * 2,
    this.bubbleRadius = bubbleRadiusDefault,
    this.bubbleTextSize = 38,
    this.bubbleTextColor = Colors.white,
    this.bubbleTailOffsetX = -10,
    required this.onDrag,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _WidgetLettersState();
}

class _WidgetLettersState extends State<WidgetLetters> {
  final _sliderKey = GlobalKey();
  int _currentIndex = -1;
  double _bubbleOffsetY = -1;

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    return Stack(
      clipBehavior: Clip.none,
      children: [
        Align(
          alignment: Alignment.centerRight,
          child: GestureDetector(
            behavior: HitTestBehavior.opaque,
            onPanDown: (details) {
              if (_sliderKey.currentContext != null && _sliderKey.currentContext!.size != null) {
                final itemSize = _sliderKey.currentContext!.size!.height / widget.letters.length;
                final testIndex = (details.localPosition.dy / itemSize).floor();
                final resIndex = testIndex < 0 ? 0 : (testIndex >= widget.letters.length ? widget.letters.length - 1 : testIndex);
                if (resIndex != _currentIndex) {
                  widget.onDrag(widget.letters[resIndex]);
                  if (mounted) {
                    setState(() {
                      _currentIndex = resIndex;
                      _bubbleOffsetY = (resIndex + 0.5) * itemSize;
                    });
                  }
                }
              }
            },
            onPanUpdate: (details) {
              if (_sliderKey.currentContext != null && _sliderKey.currentContext!.size != null) {
                final itemSize = _sliderKey.currentContext!.size!.height / widget.letters.length;
                final testIndex = (details.localPosition.dy / itemSize).floor();
                final resIndex = testIndex < 0 ? 0 : (testIndex >= widget.letters.length ? widget.letters.length - 1 : testIndex);
                if (resIndex != _currentIndex) {
                  widget.onDrag(widget.letters[resIndex]);
                  if (mounted) {
                    setState(() {
                      _currentIndex = resIndex;
                      _bubbleOffsetY = (resIndex + 0.5) * itemSize;
                    });
                  }
                }
              }
            },
            onPanEnd: (details) {
              if (mounted) {
                setState(() {
                  _currentIndex = -1;
                });
              }
            },
            onPanCancel: () {
              if (mounted) {
                setState(() {
                  _currentIndex = -1;
                });
              }
            },
            child: Container(
              key: _sliderKey,
              alignment: Alignment.center,
              width: widget.sliderWidth,
              color: widget.sliderColor,
              child: Column(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                mainAxisSize: MainAxisSize.max,
                children: _buildLetters(themeData),
              ),
            ),
          ),
        ),
        _buildBubble(themeData),
      ],
    );
  }

  List<Widget> _buildLetters(ThemeData themeData) {
    final children = <Widget>[];
    for (int i = 0; i < widget.letters.length; i++) {
      children.add(
        Text(
          widget.letters[i],
          style: i == _currentIndex ? themeData.textTheme.caption?.copyWith(color: themeData.colorScheme.secondary) : themeData.textTheme.caption,
        ),
      );
    }
    return children;
  }

  Widget _buildBubble(ThemeData themeData) {
    if (_currentIndex < 0) {
      return const SizedBox();
    } else {
      return Positioned(
        top: _bubbleOffsetY - widget.bubbleHeight / 2,
        right: widget.sliderWidth,
        child: Stack(
          children: <Widget>[
            CustomPaint(
              size: Size(widget.bubbleWidth, widget.bubbleHeight),
              painter: _WidgetLettersBubblePainter(themeData.colorScheme.primary, widget.bubbleRadius, widget.bubbleTailOffsetX),
            ),
            Container(
              width: widget.bubbleHeight,
              height: widget.bubbleHeight,
              alignment: Alignment.center,
              child: Text(widget.letters[_currentIndex], style: TextStyle(color: widget.bubbleTextColor, fontSize: widget.bubbleTextSize)),
            )
          ],
        ),
      );
    }
  }
}

class _WidgetLettersBubblePainter extends CustomPainter {
  final Color color;
  final double radius;
  final double tailOffsetX;

  _WidgetLettersBubblePainter(this.color, this.radius, this.tailOffsetX);

  @override
  void paint(Canvas canvas, Size size) {
    //画笔
    final paint = Paint();
    paint.isAntiAlias = true; //抗锯齿
    paint.style = PaintingStyle.fill;
    paint.color = color;
    //圆角矩形
    canvas.drawRRect(RRect.fromLTRBR(0, 0, size.height, size.height, Radius.circular(radius)), paint);
    //三角形
    final path = Path();
    path.moveTo(size.height + tailOffsetX, size.height * 1 / 3);
    path.lineTo(size.width, size.height / 2);
    path.lineTo(size.height + tailOffsetX, size.height * 2 / 3);
    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => false;
}
