import 'dart:io';
import 'dart:math' as math;
import 'dart:typed_data';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:image/image.dart' as imagetool;
import 'package:image_picker/image_picker.dart';
import 'package:extended_image/extended_image.dart';

///
///图片编辑器类型
///
enum WidgetPictureType {
  ///预览模式
  viewer,

  ///编辑图片，并导出为jpg格式
  editAsJpg,

  ///编辑图片，并导出为png格式
  editAsPng,

  ///编辑图片，并导出为gif格式
  editAsGif,
}

///
///图片编辑器当前状态类型
///
enum WidgetPictureMode {
  ///裁剪状态
  crop,

  ///拖拽状态
  touch,

  ///矩形涂鸦状态
  rect,

  ///圆形涂鸦状态
  circle,
}

///
///图片编辑器
///
class WidgetPicture extends StatefulWidget {
  static const mediaTypeJpg = 'image/jpg';
  static const mediaTypePng = 'image/png';
  static const mediaTypeGif = 'image/gif';

  final WidgetPictureType type;
  final List<String>? urlList;
  final List<String>? heroList;
  final String Function(String val)? onFormatUrl;
  final String? editTitle;
  final Uint8List? editBytes;
  final bool cropOnly;
  final double cropAspectRatio;
  final int cropJpgQuality;
  final int cropPngLevel;
  final int cropGifFactor;
  final bool outputResize;
  final int outputResizeWidth;
  final int outputResizeHeight;
  final Color graffitiMaskColor;
  final Color graffitiLineColor;
  final double graffitiThickness;
  final double minToolBarPaddingBottom;
  final int? initialPage;
  final ScrollPhysics? scrollPhysics;

  const WidgetPicture({
    Key? key,
    required this.type,
    this.urlList,
    this.heroList,
    this.onFormatUrl,
    this.editTitle,
    this.editBytes,
    this.cropOnly = false,
    this.cropAspectRatio = CropAspectRatios.ratio1_1,
    this.cropJpgQuality = 70,
    this.cropPngLevel = 6,
    this.cropGifFactor = 10,
    this.outputResize = false,
    this.outputResizeWidth = 720,
    this.outputResizeHeight = 720,
    this.graffitiThickness = 3,
    this.graffitiMaskColor = const Color(0x382196F3),
    this.graffitiLineColor = Colors.red,
    this.minToolBarPaddingBottom = 16,
    this.initialPage,
    this.scrollPhysics,
  })  : assert((type == WidgetPictureType.viewer && urlList != null && onFormatUrl != null) || (type != WidgetPictureType.viewer && editTitle != null && editBytes != null)),
        assert(graffitiThickness > 0),
        super(key: key);

  @override
  State<StatefulWidget> createState() => _WidgetPictureState();

  static final _imagePicker = ImagePicker(); //全局图片选择器

  static Future<Uint8List?> pickFromGallery(BuildContext context, {int width = 1280, int height = 1280, int quality = 70}) async {
    if (kIsWeb || Platform.isAndroid || Platform.isIOS) {
      final image = await _imagePicker.pickImage(source: ImageSource.gallery, maxWidth: width.toDouble(), maxHeight: height.toDouble(), imageQuality: quality); //imageQuality越小越模糊，
      return image?.readAsBytes();
    }
    final result = await FilePicker.platform.pickFiles(
      type: FileType.image,
      allowMultiple: false,
      allowCompression: true,
      withData: true,
      lockParentWindow: true,
    );
    if (result != null && result.files.isNotEmpty) {
      Uint8List? bytes;
      if (result.files.first.bytes != null) {
        bytes = result.files.first.bytes;
      } else if (result.files.first.path != null) {
        bytes = File(result.files.first.path!).readAsBytesSync();
      }
      //压缩
      if (bytes != null) {
        final minbytes = await encodePictureBytes(WidgetPictureType.editAsJpg, bytes, resize: true);
        return minbytes == null ? null : Uint8List.fromList(minbytes);
      }
    }
    return null;
  }

  static Future<Uint8List?> pickFromCamera(BuildContext context, {int width = 1280, int height = 1280, int quality = 70}) async {
    if (kIsWeb || Platform.isAndroid || Platform.isIOS) {
      final image = await _imagePicker.pickImage(source: ImageSource.camera, maxWidth: width.toDouble(), maxHeight: height.toDouble(), imageQuality: quality); //imageQuality越小越模糊
      return image?.readAsBytes();
    }
    return pickFromGallery(context, width: width, height: height, quality: quality);
  }

  static Future<List<int>?> encodePictureBytes(WidgetPictureType type, List<int> data, {int width = 1280, int height = 1280, int jpgQuality = 70, int pngLevel = 6, int gifFactor = 10, bool resize = false, bool isolate = true}) async {
    final params = {'type': type, 'data': data, 'width': width, 'height': height, 'jpgQuality': jpgQuality, 'pngLevel': pngLevel, 'gifFactor': gifFactor, 'resize': resize};
    if (isolate) {
      return await compute(_encodePictureBytesHandler, params);
    } else {
      return _encodePictureBytesHandler(params);
    }
  }

  static List<int>? _encodePictureBytesHandler(Map<String, dynamic> params) {
    final WidgetPictureType type = params['type'];
    final List<int> data = params['data'];
    final int width = params['width'];
    final int height = params['height'];
    final int jpgQuality = params['jpgQuality'];
    final int pngLevel = params['pngLevel'];
    final int gifFactor = params['gifFactor'];
    final bool resize = params['resize'];
    imagetool.Image? image = imagetool.decodeImage(data);
    if (image == null) return null;
    if (resize) {
      final scale = math.min(width / image.width, height / image.height);
      if (scale < 1) {
        //需要等比缩放
        final resizeWidth = math.min((image.width * scale).toInt(), width);
        final resizeHeight = math.min((image.height * scale).toInt(), height);
        image = imagetool.copyResize(image, width: resizeWidth, height: resizeHeight);
      }
    }
    switch (type) {
      case WidgetPictureType.editAsJpg:
        return imagetool.encodeJpg(image, quality: jpgQuality);
      case WidgetPictureType.editAsPng:
        return imagetool.encodePng(image, level: pngLevel);
      case WidgetPictureType.editAsGif:
        return imagetool.encodeGif(image, samplingFactor: gifFactor);
      default:
        return null;
    }
  }
}

class _WidgetPictureState extends State<WidgetPicture> {
  static const _debugGraffiti = false; //是否显示涂鸦的绘制层
  final _editorKey = GlobalKey<ExtendedImageEditorState>();
  final _guestureKey = GlobalKey<ExtendedImageGestureState>();
  ExtendedPageController? _pageController;
  WidgetPictureMode _currMode = WidgetPictureMode.crop;
  Uint8List? _currBytes;

  final List<_WidgetPictureGraffitiNode> _graffitiPath = []; //涂鸦位置相对于原始图片的路径
  Rect? _graffitiRect; //涂鸦图片当前的显示矩形
  double _graffitiScale = 0; //涂鸦图片相对于_graffitiRect的缩放比例
  Offset? _graffitiBegin; //涂鸦区域开始位置（相对于屏幕左上角的坐标）
  Offset? _graffitiUpdate; //涂鸦区域最新位置（相对于屏幕左上角的坐标）
  bool _saving = false;

  @override
  void initState() {
    super.initState();
    if (widget.type == WidgetPictureType.viewer) {
      _pageController = ExtendedPageController(initialPage: widget.initialPage ?? 0);
    } else {
      _currBytes = widget.editBytes;
      _currMode = widget.cropOnly ? WidgetPictureMode.crop : WidgetPictureMode.touch;
    }
  }

  @override
  void dispose() {
    _pageController?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final themeData = Theme.of(context);
    final queryData = MediaQuery.of(context);
    if (widget.type == WidgetPictureType.viewer) {
      //图片展示
      return Scaffold(
        body: GestureDetector(
          behavior: HitTestBehavior.opaque,
          child: ExtendedImageGesturePageView.builder(
            reverse: true,
            controller: _pageController,
            scrollDirection: Axis.horizontal,
            physics: widget.scrollPhysics,
            itemCount: widget.urlList!.length,
            canScrollPage: (details) => widget.urlList!.length > 1,
            itemBuilder: (BuildContext context, int index) {
              if (widget.heroList != null && widget.heroList!.length == widget.urlList!.length) {
                return ClipRect(
                  child: Hero(
                    // 解决fit不一样时，动画在收起时变化的问题
                    flightShuttleBuilder: (BuildContext flightContext, animation, HeroFlightDirection flightDirection, BuildContext fromHeroContext, BuildContext toHeroContext) {
                      return flightDirection == HeroFlightDirection.pop ? fromHeroContext.widget : toHeroContext.widget;
                    },
                    tag: widget.heroList![index],
                    child: ExtendedImage.network(
                      widget.onFormatUrl == null ? widget.urlList![index] : widget.onFormatUrl!(widget.urlList![index]),
                      fit: BoxFit.contain,
                      mode: ExtendedImageMode.gesture,
                      initGestureConfigHandler: (imagestate) => GestureConfig(inPageView: true, minScale: 1, initialScale: 1),
                    ),
                  ),
                );
              } else {
                return ClipRect(
                  child: ExtendedImage.network(
                    widget.onFormatUrl == null ? widget.urlList![index] : widget.onFormatUrl!(widget.urlList![index]),
                    fit: BoxFit.contain,
                    mode: ExtendedImageMode.gesture,
                    initGestureConfigHandler: (imagestate) => GestureConfig(inPageView: true, minScale: 1, initialScale: 1),
                  ),
                );
              }
            },
          ),
          onTap: () => Navigator.pop(context),
        ),
      );
    } else {
      //图片编辑
      return Scaffold(
        appBar: AppBar(
          title: Text(widget.editTitle ?? ''),
          actions: [IconButton(icon: const Icon(Icons.done), onPressed: () => _onSaveBtnPressed(queryData))],
        ),
        body: ClipRect(
          child: Stack(
            children: [
              Positioned(
                top: 0,
                left: 0,
                right: 0,
                bottom: 0,
                child: ExtendedImage.memory(
                  _currBytes!,
                  fit: BoxFit.contain,
                  mode: _currMode == WidgetPictureMode.crop ? ExtendedImageMode.editor : ExtendedImageMode.gesture,
                  extendedImageEditorKey: _editorKey,
                  extendedImageGestureKey: _guestureKey,
                  initGestureConfigHandler: (imagestate) => GestureConfig(inPageView: false, minScale: 1, initialScale: 1),
                  initEditorConfigHandler: (state) {
                    return EditorConfig(
                      maxScale: 8,
                      cornerColor: themeData.colorScheme.secondary,
                      cropAspectRatio: widget.cropOnly ? widget.cropAspectRatio : CropAspectRatios.custom,
                      initCropRectType: InitCropRectType.layoutRect,
                      editorMaskColorHandler: (context, down) => themeData.scaffoldBackgroundColor.withOpacity(0.4),
                    );
                  },
                  afterPaintImage: (canvas, rect, image, paint) {
                    //计算缩放比例
                    _graffitiRect = rect;
                    _graffitiScale = rect.width / image.width;
                    //初始化画笔
                    final paint = Paint();
                    paint.isAntiAlias = true; //抗锯齿
                    paint.style = PaintingStyle.stroke;
                    paint.color = widget.graffitiLineColor;
                    paint.strokeWidth = widget.graffitiThickness;
                    //绘制调试背景
                    if (_currMode == WidgetPictureMode.rect || _currMode == WidgetPictureMode.circle) {
                      canvas.drawRect(rect, Paint()..color = widget.graffitiMaskColor);
                    }
                    //绘制涂鸦路径
                    for (var element in _graffitiPath) {
                      if (element.mode == WidgetPictureMode.rect) {
                        canvas.drawRect(
                          Rect.fromLTWH(
                            element.rect.left * _graffitiScale + rect.left,
                            element.rect.top * _graffitiScale + rect.top,
                            element.rect.width * _graffitiScale,
                            element.rect.height * _graffitiScale,
                          ),
                          paint,
                        );
                      } else if (element.mode == WidgetPictureMode.circle) {
                        canvas.drawCircle(
                          Offset(
                            (element.rect.left + element.rect.width * 0.5) * _graffitiScale + rect.left,
                            (element.rect.top + element.rect.height * 0.5) * _graffitiScale + rect.top,
                          ),
                          math.min(element.rect.width, element.rect.height) * 0.5 * _graffitiScale,
                          paint,
                        );
                      }
                    }
                    //绘制当前涂鸦
                    if (_graffitiBegin != null && _graffitiUpdate != null) {
                      final x = math.min(_graffitiBegin!.dx, _graffitiUpdate!.dx);
                      final y = math.min(_graffitiBegin!.dy, _graffitiUpdate!.dy);
                      final w = (_graffitiBegin!.dx - _graffitiUpdate!.dx).abs();
                      final h = (_graffitiBegin!.dy - _graffitiUpdate!.dy).abs();
                      if (_currMode == WidgetPictureMode.rect) {
                        canvas.drawRect(Rect.fromLTWH(x, y, w, h), paint);
                      } else if (_currMode == WidgetPictureMode.circle) {
                        canvas.drawCircle(Offset(x + w * 0.5, y + h * 0.5), math.min(w, h) * 0.5, paint);
                        paint.color = widget.graffitiLineColor.withOpacity(0.5);
                        canvas.drawRect(Rect.fromLTWH(x, y, w, h), paint); //绘制范围
                      }
                    }
                  },
                ),
              ),
              Positioned(
                left: 0,
                top: 0,
                right: 0,
                bottom: 0,
                child: _currMode == WidgetPictureMode.crop || _currMode == WidgetPictureMode.touch ? const SizedBox() : _buildGraffitiHandler(),
              ),
            ],
          ),
        ),
        bottomNavigationBar: BottomAppBar(
          child: Padding(
            padding: EdgeInsets.only(bottom: math.max(queryData.padding.bottom, widget.minToolBarPaddingBottom)),
            child: Row(mainAxisAlignment: MainAxisAlignment.spaceEvenly, children: _buildBottomAppBarItems(themeData)),
          ),
        ),
      );
    }
  }

  Widget _buildGraffitiHandler() {
    return GestureDetector(
      behavior: HitTestBehavior.opaque,
      child: _debugGraffiti ? Container(color: Colors.blue.withOpacity(0.5)) : null,
      onPanDown: (details) {
        if (_graffitiRect == null) return;
        _graffitiBegin = Offset(details.globalPosition.dx, details.globalPosition.dy);
      },
      onPanUpdate: (details) {
        if (_graffitiRect == null || _graffitiBegin == null) return;
        if (mounted) {
          setState(() {
            _graffitiUpdate = Offset(details.globalPosition.dx, details.globalPosition.dy);
          });
        }
      },
      onPanEnd: (details) {
        if (_graffitiRect == null || _graffitiBegin == null || _graffitiUpdate == null) return;
        final x = (math.min(_graffitiBegin!.dx, _graffitiUpdate!.dx) - _graffitiRect!.left) / _graffitiScale;
        final y = (math.min(_graffitiBegin!.dy, _graffitiUpdate!.dy) - _graffitiRect!.top) / _graffitiScale;
        final w = (_graffitiBegin!.dx - _graffitiUpdate!.dx).abs() / _graffitiScale;
        final h = (_graffitiBegin!.dy - _graffitiUpdate!.dy).abs() / _graffitiScale;
        if (mounted) {
          setState(() {
            final minsize = widget.graffitiThickness * 2;
            if (w > minsize && h > minsize) _graffitiPath.add(_WidgetPictureGraffitiNode(_currMode, Rect.fromLTWH(x, y, w, h)));
            _graffitiBegin = null;
            _graffitiUpdate = null;
          });
        }
      },
      onPanCancel: () {
        if (_graffitiRect == null || _graffitiBegin == null || _graffitiUpdate == null) return;
        final x = (math.min(_graffitiBegin!.dx, _graffitiUpdate!.dx) - _graffitiRect!.left) / _graffitiScale;
        final y = (math.min(_graffitiBegin!.dy, _graffitiUpdate!.dy) - _graffitiRect!.top) / _graffitiScale;
        final w = (_graffitiBegin!.dx - _graffitiUpdate!.dx).abs() / _graffitiScale;
        final h = (_graffitiBegin!.dy - _graffitiUpdate!.dy).abs() / _graffitiScale;
        if (mounted) {
          setState(() {
            final minsize = widget.graffitiThickness * 2;
            if (w > minsize && h > minsize) _graffitiPath.add(_WidgetPictureGraffitiNode(_currMode, Rect.fromLTWH(x, y, w, h)));
            _graffitiBegin = null;
            _graffitiUpdate = null;
          });
        }
      },
    );
  }

  List<Widget> _buildBottomAppBarItems(ThemeData themeData) {
    if (_currMode == WidgetPictureMode.crop) {
      return [
        widget.cropOnly
            ? const SizedBox()
            : IconButton(
                icon: const Icon(Icons.close),
                onPressed: () {
                  _editorKey.currentState?.reset();
                  _setCurrentModeAndRefresh(WidgetPictureMode.touch);
                },
              ),
        IconButton(
          icon: const Icon(Icons.rotate_left),
          onPressed: () => _editorKey.currentState?.rotate(right: false),
        ),
        IconButton(
          icon: const Icon(Icons.flip),
          onPressed: () => _editorKey.currentState?.flip(),
        ),
        IconButton(
          icon: const Icon(Icons.rotate_right),
          onPressed: () => _editorKey.currentState?.rotate(right: true),
        ),
        widget.cropOnly
            ? const SizedBox()
            : IconButton(
                icon: const Icon(Icons.done),
                onPressed: () {
                  _cropFlipRotatePicture(setBytes: true); //reset()前执行
                  _editorKey.currentState?.reset();
                  _setCurrentModeAndRefresh(WidgetPictureMode.touch);
                },
              ),
      ];
    } else {
      return [
        IconButton(
          icon: const Icon(Icons.crop),
          //限制只能裁剪一次，不然无限裁剪到一定次数会出问题
          onPressed: _currBytes == widget.editBytes
              ? () {
                  _graffitiPath.clear();
                  _guestureKey.currentState?.reset();
                  _setCurrentModeAndRefresh(WidgetPictureMode.crop);
                }
              : null,
        ),
        IconButton(
          icon: Icon(Icons.swipe_outlined, color: _currMode == WidgetPictureMode.touch ? themeData.colorScheme.secondary : null),
          onPressed: () => _setCurrentModeAndRefresh(WidgetPictureMode.touch),
        ),
        IconButton(
          icon: Icon(Icons.crop_din_outlined, color: _currMode == WidgetPictureMode.rect ? themeData.colorScheme.secondary : null),
          onPressed: () => _setCurrentModeAndRefresh(WidgetPictureMode.rect),
        ),
        IconButton(
          icon: Icon(Icons.circle_outlined, color: _currMode == WidgetPictureMode.circle ? themeData.colorScheme.secondary : null),
          onPressed: () => _setCurrentModeAndRefresh(WidgetPictureMode.circle),
        ),
        IconButton(
          icon: const Icon(Icons.restore_outlined),
          onPressed: () {
            _currBytes = widget.editBytes; //重置为初始数据
            _graffitiPath.clear();
            _guestureKey.currentState?.reset();
            _setCurrentModeAndRefresh(WidgetPictureMode.touch);
          },
        ),
      ];
    }
  }

  void _setCurrentModeAndRefresh(WidgetPictureMode mode) {
    if (mounted) {
      setState(() {
        _currMode = mode;
      });
    }
  }

  imagetool.Image? _cropFlipRotatePicture({required bool setBytes}) {
    //转换
    imagetool.Image? image = imagetool.decodeImage(_currBytes!);
    if (image != null) {
      //编辑
      final editAction = _editorKey.currentState?.editAction;
      if (editAction != null) {
        // print('---->editAction');
        //裁剪
        final cropRect = _editorKey.currentState?.getCropRect();
        if (cropRect != null) {
          // print('---->裁剪');
          image = imagetool.copyCrop(image, cropRect.left.toInt(), cropRect.top.toInt(), cropRect.width.toInt(), cropRect.height.toInt());
        }
        //翻转
        if (editAction.needFlip) {
          // print('---->翻转');
          if (editAction.flipY && editAction.flipX) {
            image = imagetool.flip(image, imagetool.Flip.both);
          } else if (editAction.flipY) {
            image = imagetool.flip(image, imagetool.Flip.horizontal);
          } else if (editAction.flipX) {
            image = imagetool.flip(image, imagetool.Flip.vertical);
          }
        }
        //旋转
        if (editAction.hasRotateAngle) {
          // print('---->旋转');
          image = imagetool.copyRotate(image, editAction.rotateAngle);
        }
        //设置字节数据
        if (setBytes) {
          // print('---->数据');
          _currBytes = Uint8List.fromList(imagetool.encodeJpg(image, quality: widget.cropJpgQuality));
        }
      }
    }
    return image;
  }

  void _onSaveBtnPressed(MediaQueryData queryData) async {
    if (_saving) return;
    _saving = true;
    //变换
    imagetool.Image? image = _cropFlipRotatePicture(setBytes: false);
    if (image == null) {
      Navigator.pop(context);
      return;
    }
    //涂鸦
    for (var element in _graffitiPath) {
      final repreat = queryData.devicePixelRatio * widget.graffitiThickness; //加粗
      if (element.mode == WidgetPictureMode.rect) {
        for (int i = 0; i < repreat; i++) {
          imagetool.drawRect(
            image,
            element.rect.left.toInt() - i,
            element.rect.top.toInt() - i,
            element.rect.right.toInt() + i,
            element.rect.bottom.toInt() + i,
            imagetool.getColor(widget.graffitiLineColor.red, widget.graffitiLineColor.green, widget.graffitiLineColor.blue, widget.graffitiLineColor.alpha),
          );
        }
      } else if (element.mode == WidgetPictureMode.circle) {
        for (int i = 0; i < repreat; i++) {
          imagetool.drawCircle(
            image,
            (element.rect.left + element.rect.width * 0.5).toInt(),
            (element.rect.top + element.rect.height * 0.5).toInt(),
            (math.min(element.rect.width, element.rect.height) * 0.5).toInt() + i,
            imagetool.getColor(widget.graffitiLineColor.red, widget.graffitiLineColor.green, widget.graffitiLineColor.blue, widget.graffitiLineColor.alpha),
          );
        }
      }
    }
    //尺寸
    if (widget.outputResize) {
      // print('---->resize');
      image = imagetool.copyResize(image, width: widget.outputResizeWidth, height: widget.outputResizeHeight); //非等比缩放
    }
    //编码
    switch (widget.type) {
      case WidgetPictureType.editAsJpg:
        Navigator.pop(context, imagetool.encodeJpg(image, quality: widget.cropJpgQuality));
        return;
      case WidgetPictureType.editAsPng:
        Navigator.pop(context, imagetool.encodePng(image, level: widget.cropPngLevel));
        return;
      case WidgetPictureType.editAsGif:
        Navigator.pop(context, imagetool.encodeGif(image, samplingFactor: widget.cropGifFactor));
        return;
      default:
        Navigator.pop(context);
        return;
    }
  }
}

class _WidgetPictureGraffitiNode {
  final WidgetPictureMode mode;
  final Rect rect;
  _WidgetPictureGraffitiNode(this.mode, this.rect);
}
