import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import '../forked/city_picker_china.dart';

///
///底部弹出文本选择框子项数据
///
class WidgetSheetItem {
  final int id;
  final String text;
  final Color? color;

  WidgetSheetItem(this.id, this.text, {this.color});
}

///
///底部弹出框
///
class WidgetSheet {
  static const cancelMargin = 8.0;
  static const contentRadius = Radius.circular(8);

  ///弹出顶部模态框
  static Future<T?> showTopChildSheet<T>(BuildContext context, {required Widget child, double? emptyFactory, double? emptyHeight}) {
    return Navigator.push(context, _WidgetSheetCustomRoute(child: child, bottom: false, emptyFactory: emptyFactory, emptyHeight: emptyHeight));
  }

  ///弹出底部模态框
  static Future<T?> showBottomChildSheet<T>(BuildContext context, {required Widget child, bool enableDrag = true, double heightFactor = 0.7}) {
    final themeData = Theme.of(context);
    return showModalBottomSheet<T>(
      context: context,
      backgroundColor: themeData.cardColor,
      isScrollControlled: true,
      clipBehavior: Clip.hardEdge,
      enableDrag: enableDrag,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: contentRadius, topRight: contentRadius)),
      builder: (context) {
        return FractionallySizedBox(heightFactor: heightFactor, child: child);
      },
    );
  }

  ///弹出文本选择框
  static Future<int?> showBottomTextSheet(BuildContext context, List<WidgetSheetItem> itemDatas, {String? cancelText}) {
    final themeData = Theme.of(context);
    final itemViews = <Widget>[];
    for (var element in itemDatas) {
      itemViews.add(ListTile(
        title: Text(element.text, textAlign: TextAlign.center, style: themeData.textTheme.subtitle1?.copyWith(color: element.color)),
        onTap: () => Navigator.pop(context, element.id),
      ));
      itemViews.add(const Divider());
    }
    if (cancelText != null && cancelText.trim() != '') {
      itemViews.add(Container(color: themeData.scaffoldBackgroundColor, height: cancelMargin));
      itemViews.add(const Divider());
      itemViews.add(ListTile(
        title: Text(cancelText, textAlign: TextAlign.center, style: themeData.textTheme.subtitle1),
        onTap: () => Navigator.pop(context),
      ));
    }
    return showModalBottomSheet<int>(
      context: context,
      backgroundColor: themeData.cardColor,
      clipBehavior: Clip.hardEdge,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: contentRadius, topRight: contentRadius)),
      builder: (context) {
        return SafeArea(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: itemViews,
          ),
        );
      },
    );
  }

  ///弹出网格选择框
  static Future<T?> showBottomGridSheet<T>(BuildContext context, List<List<T>> itemDatas, {required Widget Function(T item) builder, ScrollPhysics? physics, double itemWidth = 90, double itemHeight = 90, String? cancelText}) {
    final themeData = Theme.of(context);
    final itemViews = <Widget>[];
    for (var row in itemDatas) {
      itemViews.add(
        SizedBox(
          height: itemHeight,
          child: ListView(
            physics: physics,
            scrollDirection: Axis.horizontal,
            children: row.map((e) => SizedBox(width: itemWidth, child: builder(e))).toList(),
          ),
        ),
      );
      itemViews.add(const Divider());
    }
    if (cancelText != null && cancelText.trim() != '') {
      itemViews.add(Container(color: themeData.scaffoldBackgroundColor, height: cancelMargin));
      itemViews.add(const Divider());
      itemViews.add(ListTile(
        title: Text(cancelText, textAlign: TextAlign.center, style: themeData.textTheme.subtitle1),
        onTap: () => Navigator.pop(context),
      ));
    }
    return showModalBottomSheet<T>(
      context: context,
      backgroundColor: themeData.cardColor,
      clipBehavior: Clip.hardEdge,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: contentRadius, topRight: contentRadius)),
      builder: (context) {
        return SafeArea(
          child: Column(
            mainAxisSize: MainAxisSize.min,
            children: itemViews,
          ),
        );
      },
    );
  }

  ///弹出日期选择框
  static void showBottomDateSheet(BuildContext context, {required void Function(DateTime value) onDateTimeChanged, CupertinoDatePickerMode mode = CupertinoDatePickerMode.date, DateTime? minimumDate, DateTime? maximumDate, DateTime? initialDateTime}) {
    final themeData = Theme.of(context);
    final queryData = MediaQuery.of(context);
    showModalBottomSheet(
      context: context,
      backgroundColor: themeData.cardColor,
      clipBehavior: Clip.hardEdge,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: contentRadius, topRight: contentRadius)),
      builder: (context) {
        return CupertinoTheme(
          data: CupertinoThemeData(brightness: themeData.brightness),
          child: SizedBox(
            height: queryData.size.height / 2.5,
            child: CupertinoDatePicker(
              mode: mode,
              minimumDate: minimumDate,
              maximumDate: maximumDate,
              initialDateTime: initialDateTime,
              onDateTimeChanged: onDateTimeChanged,
            ),
          ),
        );
      },
    );
  }

  ///弹出城市选择框
  static Future<CityResult?> showBottomCitySheet(BuildContext context, {required String province, required String city, required String district}) {
    final themeData = Theme.of(context);
    final queryData = MediaQuery.of(context);
    return showModalBottomSheet(
      context: context,
      backgroundColor: themeData.cardColor,
      clipBehavior: Clip.hardEdge,
      shape: const RoundedRectangleBorder(borderRadius: BorderRadius.only(topLeft: contentRadius, topRight: contentRadius)),
      builder: (context) {
        return CupertinoTheme(
          data: CupertinoThemeData(brightness: themeData.brightness),
          child: SizedBox(
            height: queryData.size.height / 2.5,
            child: CityPicker.fromName(province: province, city: city, county: district),
          ),
        );
      },
    );
  }
}

class _WidgetSheetCustomRoute<T> extends PopupRoute<T> {
  final Widget child;
  final bool bottom;
  final double? emptyFactory;
  final double? emptyHeight;

  _WidgetSheetCustomRoute({required this.child, required this.bottom, this.emptyFactory, this.emptyHeight});

  @override
  Duration get transitionDuration => const Duration(milliseconds: 300);

  @override
  Color get barrierColor => Colors.black45;

  @override
  bool get barrierDismissible => false;

  @override
  String get barrierLabel => '';

  @override
  Widget buildPage(BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation) {
    final queryData = MediaQuery.of(context);
    final bodyHeight = queryData.size.height - (emptyHeight ?? ((emptyFactory ?? 0.3) * queryData.size.height));
    return SlideTransition(
      position: Tween(begin: bottom ? const Offset(0.0, 1.0) : const Offset(0.0, -1.0), end: Offset.zero).chain(CurveTween(curve: Curves.easeInOut)).animate(animation),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          bottom ? const SizedBox() : SizedBox(height: bodyHeight, child: child),
          Expanded(child: GestureDetector(behavior: HitTestBehavior.opaque, child: Container(color: Colors.transparent), onTap: () => Navigator.pop(context))),
          bottom ? SizedBox(height: bodyHeight, child: child) : const SizedBox(),
        ],
      ),
    );
  }
}
